/// <reference path="typings/index.d.ts" />
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const chalk = require("chalk");
const readline = require("readline");
function error(msg) {
    console.log(chalk.red(`Error: ${msg}`));
}
function isPromise(val) {
    return val instanceof Object && typeof val.then === 'function';
}
class REPL {
    constructor(options) {
        this.lastError = null;
        options = options || {};
        this.byeMessage = options.byeMessage || 'Bye.';
    }
    start() {
        if (this.rl)
            throw new Error(`REPL already started`);
        this.rl = readline.createInterface({
            input: process.stdin,
            output: process.stdout
        });
        const read = () => __awaiter(this, void 0, void 0, function* () {
            this.rl.question(' > ', (answer) => {
                try {
                    const chunks = answer.split(' '), command = chunks[0];
                    switch (command) {
                        case 'stack':
                            if (this.lastError === null)
                                error('No recent errors.');
                            else
                                console.log(this.lastError.stack);
                            read();
                            break;
                        case 'quit':
                            console.log(this.byeMessage);
                            process.exit();
                        case 'alias':
                            const aliasName = chunks[1];
                            if (!aliasName)
                                throw new Error(`must provide an alias name`);
                            if (chunks.length < 3)
                                throw new Error(`must provide a command`);
                            this.aliases[aliasName] = chunks.slice(2).join(' ');
                            console.log(chalk.green(`Alias '${aliasName}' created.`));
                            break;
                        default:
                            const res = this.evaluate(answer);
                            if (isPromise(res))
                                res.then(() => read())
                                    .catch(e => {
                                    this.lastError = e;
                                    error(e.message);
                                    read();
                                });
                    }
                }
                catch (e) {
                    this.lastError = e;
                    error(e.message);
                    read();
                }
            });
        });
        read().catch(e => error(e.stack));
    }
    stop() {
        if (!this.rl)
            throw new Error(`REPL not started`);
        this.rl.close();
    }
}
exports.REPL = REPL;
